// INCLUDES ///////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN  

// you must #define INITGUID if not done elsewhere
#define INITGUID

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <objbase.h>
#include <iostream> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>
#include <fstream>
#include <dsound.h>

#include <ddraw.h>     // directX includes
#include "T3DLIB1.H"

using namespace std;
// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINCLASS1"

#define TEST	false //If testing skip intro and stuff

// default screen size
#define SCREEN_WIDTH    640  // size of screen
#define SCREEN_HEIGHT   480
#define SCREEN_BPP      8    // bits per pixel
#define BITMAP_ID            0x4D42 // universal id for a bitmap
#define MAX_COLORS_PALETTE   256

// used to flag type of object
#define BLANK_SPACE		0
#define ROCK_OBJ		1
#define FIREFLY_OBJ		2
#define BEE_OBJ			3
#define SCORPION_OBJ	4
#define TERMITE_OBJ		5
#define SPIDER_WEB_OBJ	6
#define DOOR_OBJ		7
#define WALL_OBJ		8
#define PLAYER_OBJ		9
#define HONEY_OBJ		10
#define BADGER_OBJ		11

//Directional constants
#define DOWN	0
#define LEFT	1
#define UP		2
#define RIGHT	3

//Some drawing constants
#define CELL_HEIGHT 70
#define CELL_WIDTH	50
#define MOVE_SPEED_X	CELL_WIDTH / 10
#define MOVE_SPEED_Y	CELL_WIDTH / 10

// TYPES ///////////////////////////////////////////////////

// need something to hold each object
typedef struct SPARSE_OBJ_TYP
{
	int type; // type of object: beacon, generator, or alien
                   // could have just used a bob_ptr here, but I like
                   // doing this the hard way so you can exactly what's happening
                   
	int x,y; // position of object 
	
} SPARSE_OBJ, *SPARSE_OBJ_PTR;

typedef struct Maze//Maze struct simulates the levels
{
	int xsize, ysize, xpos, ypos;
	SPARSE_OBJ_TYP maze[200][200];
};
Maze currentMaze;

// PROTOTYPES /////////////////////////////////////////////

// game console
int Game_Init(void *parms=NULL, int num_parms = 0);
int Game_Shutdown(void *parms=NULL, int num_parms = 0);
int Game_Main(void *parms=NULL,  int num_parms = 0);

// GLOBALS ////////////////////////////////////////////////

// windows vars
HWND      main_window_handle = NULL; // globally track main window
int       window_closed      = 0;    // tracks if window is closed
HINSTANCE main_instance      = NULL; // globally track hinstance

// BOB globals
BOB	player;   // the player BOB
BOB base_rock; //Base rock
BOB base_scorpion; //Base scorpion
BOB base_bee; //Base bee
BOB	base_termite; //Base termite
BOB base_firefly; //Base firefly
BOB base_door; //Base door
BOB base_spiderweb; //Base spiderweb
BOB base_honey;
BOB base_badger;
BOB base_wall;
BOB explosion;

BITMAP_IMAGE steve; //Steve Zissou intro
BITMAP_IMAGE credits; //End game credits
BITMAP_IMAGE lose_img; //Lose game image
BITMAP_IMAGE how_to; //Begining tutorial

int world_x = -SCREEN_WIDTH / 2 +  .5 * CELL_WIDTH,   // current position of viewing window
    world_y = -SCREEN_HEIGHT / 2 + .5 * CELL_HEIGHT;

//This array holds the bugs for the hotbar
int BUGS[] = {BLANK_SPACE, BLANK_SPACE, BLANK_SPACE, BLANK_SPACE};;

//This is the value for the player lives
int lives;

char mazeName[10]; //Used to load in the maze
int mazeNumber = 1; //The current level #

bool boom = false; //Used to detect if something's been removed

RECT dest_rect; //Used to draw a rectangle behind text as to make it easier to read

int direction; //The player's current direction
bool update; //Weather or not to update the player animation

ofstream debug("debug.txt"); //A debug text file as cout cannot be used for debugging

// PROTOTYPES //////////////////////////////////////////////

LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT		ps;		// used in WM_PAINT
HDC				hdc;	// handle to a device context

// what is the message 
switch(msg)
	{	
	case WM_CREATE: 
        {
		// do initialization stuff here

        // return success
		return(0);
		} break;
   
	case WM_PAINT: 
		{
		// simply validate the window 
   	    hdc = BeginPaint(hwnd,&ps);	 
        
        // end painting
        EndPaint(hwnd,&ps);

        // return success
		return(0);
   		} break;

	case WM_DESTROY: 
		{

		// kill the application, this sends a WM_QUIT message 
		PostQuitMessage(0);

        // return success
		return(0);
		} break;

	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc

// WINMAIN ////////////////////////////////////////////////


//This function draws a rectangle of the given color and size
int draw_rect(int left, int right, int top, int bottom, int color)
{
	ddbltfx.dwFillColor = color; //add the proper values to dest_rect and the blitter object
	dest_rect.left   = left;
	dest_rect.top    = top;
	dest_rect.right  = right;
	dest_rect.bottom = bottom;

		// make the blitter call
	if (FAILED(lpddsprimary->Blt(&dest_rect, // pointer to dest RECT
		NULL, // pointer to source surface
		NULL, // pointer to source RECT
		DDBLT_COLORFILL | DDBLT_WAIT, 
		// do a color fill and wait if you have to
		&ddbltfx))) // pointer to DDBLTFX holding info
			return(0); //Failure

	return 1; //Success
}


//This method is called when the player wins the game
void win()
{
	//Draw a rectangle before the text so it's easier to read
	draw_rect(SCREEN_WIDTH / 2 - 50, SCREEN_WIDTH / 2 + 75, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
	Draw_Text_GDI("You win!", SCREEN_WIDTH / 2 - 25, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
	Sleep(1000); //Pause to let the player read the text

	//Draw in the credits
	DDraw_Lock_Primary_Surface();
	Draw_Bitmap(&credits, primary_buffer, primary_lpitch,0);
	DDraw_Unlock_Primary_Surface();
	Sleep(3000);

	//End the game
	PostMessage(main_window_handle, WM_DESTROY,0,0);
}


//This function loads a maze(level) into the game
inline void LoadMaze()
{
	for(int a = 0; a < 4; a++) //Clear out the bug hotbar array in case there was stuff there
		BUGS[a] = BLANK_SPACE;

	sprintf(mazeName,"maze%d.txt",mazeNumber); //Compute the proper filename
	ifstream infile(mazeName); //Load up the maze file
	
	if(mazeNumber == 4) //We only have 3 levels, so if it's level 4, call win()
		win();

	infile >> currentMaze.ysize; //Read in the y size
	infile >> currentMaze.xsize; //Read in the x size
	
	for(int a = 0; a < currentMaze.ysize; a++) //load up the maze from the file
	{
		for(int b = 0; b < currentMaze.xsize; b++)
		{
			infile >> currentMaze.maze[a][b].type;
			if(currentMaze.maze[a][b].type == PLAYER_OBJ){ //Make the player the center
				currentMaze.ypos = a;
				currentMaze.xpos = b;
				currentMaze.maze[a][b].type = BLANK_SPACE; //The nuke him so he can't collide with himself
			}else if(currentMaze.maze[a][b].type != BLANK_SPACE){ //If it's a non blank object, compute the x,y
				currentMaze.maze[a][b].x = b * CELL_WIDTH - CELL_WIDTH;
				currentMaze.maze[a][b].y = a * CELL_HEIGHT - CELL_HEIGHT;
			}
		}
	}

	infile.close(); //Close the maze file
}


//This function adds a bug to the bug hotbar array
void addBug(int bug)
{
	for(int a = 0; a < 4; a++){ //Can't have more than one of the same bug
		if(BUGS[a] == bug)
			return;

		if(BUGS[a] == BLANK_SPACE){ //First blank space is where we put the bug
			BUGS[a] = bug;
			return;
		}
	}
}


//This function matches a bug to the object it 'defeats'
bool match(int bug, int object){
	if(bug == FIREFLY_OBJ && object == SPIDER_WEB_OBJ) //Firefly > spiderweb
		return true;

	if(bug == TERMITE_OBJ && object == WALL_OBJ) //Termite > wall
		return true;

	if(bug == BEE_OBJ && object == HONEY_OBJ) //Bee > honeycomb
		return true;

	if(bug == SCORPION_OBJ && object == BADGER_OBJ) //Scorpion > badger
		return true;

	return false; //No match
}



//This function is called when the player loses
void lose()
{
	//Draw the lose image
	DDraw_Lock_Primary_Surface();
	Draw_Bitmap(&lose_img, primary_buffer, primary_lpitch,0);
	DDraw_Unlock_Primary_Surface();
	Sleep(3000);

	//Draw the credits
	DDraw_Lock_Primary_Surface();
	Draw_Bitmap(&credits, primary_buffer, primary_lpitch,0);
	DDraw_Unlock_Primary_Surface();
	Sleep(3000);

	//End the game
	PostMessage(main_window_handle, WM_DESTROY,0,0);
}


//This function is called when the player is killed
void lose_life()
{
	lives--; //Decrement the life counter

	if(lives == 0) //No more lives? game over
		lose();

	LoadMaze(); //Reload the current maze
}


//This function prints 
void printBugInfo(int bug)
{
	int sleep_time = 1500; //Use a constant sleep time for every message

	if(bug == FIREFLY_OBJ){ //Each message is dependant on the bug being used
		draw_rect(SCREEN_WIDTH / 2 - 130, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
		Draw_Text_GDI("The firefly can burn through spiderwebs!", SCREEN_WIDTH / 2 - 125, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
		Sleep(sleep_time);
	}else if(bug == TERMITE_OBJ){
		draw_rect(SCREEN_WIDTH / 2 - 130, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
		Draw_Text_GDI("The termite chews through walls!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
		Sleep(sleep_time);
	}else if(bug == BEE_OBJ){
		draw_rect(SCREEN_WIDTH / 2 - 130, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
		Draw_Text_GDI("The bee can collect honeycombs!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
		Sleep(sleep_time);
	}else if(bug == SCORPION_OBJ){
		draw_rect(SCREEN_WIDTH / 2 - 130, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
		Draw_Text_GDI("The scorpion kills badgers!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
		Sleep(sleep_time);
	}

}


//This function shifts the y coords of the world by move
void moveWorldY(int move)
{
	for(int a = 0; a < currentMaze.ysize; a++) //Double for loop to get boy x and y
		for(int b = 0; b < currentMaze.xsize; b++)
			if(currentMaze.maze[a][b].type != PLAYER_OBJ)
				currentMaze.maze[a][b].y += move; //add move to the y value
}


//This function shifts the x coords of the world by move
void moveWorldX(int move)
{
	for(int a = 0; a < currentMaze.ysize; a++) //Double for loop for both x and y
		for(int b = 0; b < currentMaze.xsize; b++)
			if(currentMaze.maze[a][b].type != PLAYER_OBJ)
				currentMaze.maze[a][b].x += move; //add move to x
}



int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{


WNDCLASSEX winclass; // this will hold the class we create
HWND	   hwnd;	 // generic window handle
MSG		   msg;		 // generic message
HDC        hdc;      // graphics device context

// first fill in the window class stucture
winclass.cbSize         = sizeof(WNDCLASSEX);
winclass.style			= CS_DBLCLKS | CS_OWNDC | 
                          CS_HREDRAW | CS_VREDRAW;
winclass.lpfnWndProc	= WindowProc;
winclass.cbClsExtra		= 0;
winclass.cbWndExtra		= 0;
winclass.hInstance		= hinstance;
winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
winclass.hCursor		= LoadCursor(NULL, IDC_ARROW); 
winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
winclass.lpszMenuName	= NULL;
winclass.lpszClassName	= WINDOW_CLASS_NAME;
winclass.hIconSm        = LoadIcon(NULL, IDI_APPLICATION);

// save hinstance in global
main_instance = hinstance;

// register the window class
if (!RegisterClassEx(&winclass))
	return(0);

// create the window
if (!(hwnd = CreateWindowEx(NULL,                  // extended style
                            WINDOW_CLASS_NAME,     // class
						    "Bug Maze!", // title
						    WS_POPUP | WS_VISIBLE,
					 	    0,0,	  // initial x,y
						    SCREEN_WIDTH,SCREEN_HEIGHT,  // initial width, height
						    NULL,	  // handle to parent 
						    NULL,	  // handle to menu
						    hinstance,// instance of this application
						    NULL)))	  // extra creation parms
return(0);

// save main window handle
main_window_handle = hwnd;

// initialize game here
Game_Init();

// enter main event loop
while(TRUE)
	{
    // test if there is a message in queue, if so get it
	if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
	   { 
	   // test if this is a quit
       if (msg.message == WM_QUIT)
           break;
	
	   // translate any accelerator keys
	   TranslateMessage(&msg);

	   // send the message to the window proc
	   DispatchMessage(&msg);
	   } // end if
    
       // main game processing goes here
       Game_Main();
       
	} // end while

// closedown game here
Game_Shutdown();

// return to Windows like this
return(msg.wParam);

} // end WinMain


// GAME PROGRAMMING CONSOLE FUNCTIONS ////////////////

int Game_Init(void *parms,  int num_parms)
{
	int index; // looping var
	direction = DOWN; //Set default direction to down
	update = true; //Update bool is true for animation
	lives = 3; //Base lives is 3

	// initialize directdraw
	DDraw_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);
	DDRAW_INIT_STRUCT(ddbltfx);

	ShowCursor(false); //get rid of the cursor

	if(!TEST){ //If testing don't load the intro, or howto, it gets annoying

		//Load up the intro image
		Load_Bitmap_File(&bitmap8bit, "WIDESCREENSTEVE8.BMP");
		Create_Bitmap(&steve,0,0,640,480);
		Load_Image_Bitmap(&steve, &bitmap8bit, 0, 0, BITMAP_EXTRACT_MODE_ABS);
		Set_Palette(bitmap8bit.palette);

		DDraw_Lock_Primary_Surface(); //Draw intro image
		Draw_Bitmap(&steve, primary_buffer, primary_lpitch,0);
		DDraw_Unlock_Primary_Surface();

		Unload_Bitmap_File(&bitmap8bit);
		Sleep(4000); //Pause so people can see the image

		Load_Bitmap_File(&bitmap8bit, "HOWTO8.BMP"); //Load up the howto image
		Create_Bitmap(&how_to,0,0,640,480);
		debug<< "yep\n";debug.close();
		Load_Image_Bitmap(&how_to, &bitmap8bit, 0, 0, BITMAP_EXTRACT_MODE_ABS);
		Set_Palette(bitmap8bit.palette);
		
		DDraw_Lock_Primary_Surface(); //Draw the how to image
		Draw_Bitmap(&how_to, primary_buffer, primary_lpitch,0);
		DDraw_Unlock_Primary_Surface();

		Unload_Bitmap_File(&bitmap8bit);
		Sleep(4000); //Pause so people can see the image
	}

	//Load up the end game credits
	Load_Bitmap_File(&bitmap8bit, "CREDITS8.BMP");
	Create_Bitmap(&credits,0,0,640,480);
	Load_Image_Bitmap(&credits, &bitmap8bit, 0, 0, BITMAP_EXTRACT_MODE_ABS);
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit);

	//Load up the lose bitmap
	Load_Bitmap_File(&bitmap8bit, "LOSE.BMP");
	Create_Bitmap(&lose_img,0,0,640,480);
	Load_Image_Bitmap(&lose_img, &bitmap8bit, 0, 0, BITMAP_EXTRACT_MODE_ABS);
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit);

	//Load the bitmap for the explosion
	Load_Bitmap_File(&bitmap8bit, "EXPLOSION8.BMP");
	Set_Palette(bitmap8bit.palette);

	//Create a BOB for the explosion
	if (!Create_BOB(&explosion, 0, 0, 30, 34, 2, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	for (index = 0; index < 2; index++) //Load up the animations
		Load_Frame_BOB(&explosion,&bitmap8bit,index,index,0,BITMAP_EXTRACT_MODE_CELL); 

	int explode[] = {0, 1, 0}; //Explosion

	Load_Animation_BOB(&explosion, 0, 3, explode); //Load in the explosion animation

	Set_Anim_Speed_BOB(&explosion, 5); //Set the speed to 5(fast)
	Set_Animation_BOB(&explosion, 0);

	Unload_Bitmap_File(&bitmap8bit);

	//Load in the player bitmap
	Load_Bitmap_File(&bitmap8bit, "ASHWALK83.BMP");
	Set_Palette(bitmap8bit.palette);

	//Create the player BOB
	if (!Create_BOB(&player,SCREEN_WIDTH / 2 - CELL_WIDTH / 2, SCREEN_HEIGHT / 2 - CELL_HEIGHT / 2,15,23,24, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	for (index = 0; index < 24; index++) //Load up the player animations
		Load_Frame_BOB(&player,&bitmap8bit,index,index,0,BITMAP_EXTRACT_MODE_CELL); 

	int stand_down[] = {0}; //Standing down sprites
	int stand_left[] = {3}; //Standing left sprites
	int stand_up[] = {6}; //Standing up sprites
	int stand_right[] = {9}; //Standing right sprites
	int walk_down[] = {1, 2, 1, 2}; //Walking down animations
	int walk_left[] = {4, 5, 4, 5}; //Walking left animations
	int walk_up[] = {7, 8, 7, 8}; //Walking up animations
	int walk_right[] = {10, 11, 10, 11}; //Walking right animations

	Load_Animation_BOB(&player, 0, 1, stand_down); //Load in each walking/standing animation array
	Load_Animation_BOB(&player, 1, 1, stand_left);
	Load_Animation_BOB(&player, 2, 1, stand_up);
	Load_Animation_BOB(&player, 3, 1, stand_right);
	Load_Animation_BOB(&player, 4, 4, walk_down);
	Load_Animation_BOB(&player, 5, 4, walk_left);
	Load_Animation_BOB(&player, 6, 4, walk_up);
	Load_Animation_BOB(&player, 7, 4, walk_right);

	Set_Anim_Speed_BOB(&player, 10); //Set the speed to 10
	Set_Animation_BOB(&player, 0);

	Unload_Bitmap_File(&bitmap8bit); //Unload the bitmap file

	Load_Bitmap_File(&bitmap8bit, "ROCK.BMP"); //Load in rock.bmp

	if (!Create_BOB(&base_rock,0, 0,16,15,1, //Create the base BOB for the rock image
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	Load_Frame_BOB(&base_rock, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL); //Load in the image
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit); //Unload rock.bmp

	Load_Bitmap_File(&bitmap8bit, "BEE8.BMP"); //Load in bee8

	if (!Create_BOB(&base_bee, 0, 0, 72, 100, 1, //Create the base BOB for the bee image
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	Load_Frame_BOB(&base_bee, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL); //Load in the image
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit); //Unload bee.bmp

	Load_Bitmap_File(&bitmap8bit, "FIREFLY8.BMP"); //Load in firefly8

	if (!Create_BOB(&base_firefly, 0, 0, 440, 788, 1, //Create the base BOB for the firefly image
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	Load_Frame_BOB(&base_firefly, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL); //Load in the image
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit); //Unload firefly.bmp

	Load_Bitmap_File(&bitmap8bit, "SCORPION8.BMP"); //Load in scorpion8

	if (!Create_BOB(&base_scorpion, 0, 0, 81, 180, 1, //Create the base BOB for the scropion image
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	Load_Frame_BOB(&base_scorpion, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL); //Load in the image
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit); //Unload scorpion.bmp

	Load_Bitmap_File(&bitmap8bit, "TERMITE8.BMP"); //Load in termite8

	if (!Create_BOB(&base_termite, 0, 0, 100, 99, 1, //Create the base BOB for the termite image
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	Load_Frame_BOB(&base_termite, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL); //Load in the image
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit); //Unload termite.bmp

	Load_Bitmap_File(&bitmap8bit, "DOOR.BMP"); //Load in door

	if (!Create_BOB(&base_door, 0, 0, 20, 20, 1, //Create the base BOB for the door image
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	Load_Frame_BOB(&base_door, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL); //Load in the image
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit); //Unload door.bmp

	Load_Bitmap_File(&bitmap8bit, "SPIDERWEB.BMP"); //Load in spiderweb

	if (!Create_BOB(&base_spiderweb, 0, 0, 20, 19, 1, //Create the base BOB for the spiderweb image
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	Load_Frame_BOB(&base_spiderweb, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL); //Load in the image
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit); //Unload spiderweb.bmp

	Load_Bitmap_File(&bitmap8bit, "HONEYCOMB8.BMP"); //Load in honeycomb8

	if (!Create_BOB(&base_honey, 0, 0, 100, 149, 1, //Create the base BOB for the honeycomb8 image
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	Load_Frame_BOB(&base_honey, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL); //Load in the image
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit); //Unload honeycomb8.bmp

	Load_Bitmap_File(&bitmap8bit, "BADGER8.BMP"); //Load in badger8

	if (!Create_BOB(&base_badger, 0, 0, 128, 128, 1, //Create the base BOB for the badger8 image
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	Load_Frame_BOB(&base_badger, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL); //Load in the image
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit); //Unload badger8.bmp

	Load_Bitmap_File(&bitmap8bit, "WALL8.BMP"); //Load in wall8

	if (!Create_BOB(&base_wall, 0, 0, 150, 224, 1, //Create the base BOB for the wall8 image
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME,DDSCAPS_SYSTEMMEMORY, 255))
	   return(0);

	Load_Frame_BOB(&base_wall, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL); //Load in the image
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit); //Unload wall8.bmp

	LoadMaze(); //Load in the first level

	//No sound in the vlt version due to size of sound files
	//PlaySound("ping.wav", main_instance, SND_ASYNC | SND_LOOP); //Start playing the background music

	// return success
	return(1);

} // end Game_Init

///////////////////////////////////////////////////////////

int Game_Shutdown(void *parms,  int num_parms)
{

	debug.close(); //close our debug file

	ShowCursor(true); //Show the cursor again

	//Destroy all the bitmaps
	Destroy_Bitmap(&steve);
	Destroy_Bitmap(&credits);
	Destroy_Bitmap(&lose_img);
	Destroy_Bitmap(&how_to);

	//Destroy all the BOBs
	Destroy_BOB(&player);
	Destroy_BOB(&base_rock);
	Destroy_BOB(&base_firefly);
	Destroy_BOB(&base_bee);
	Destroy_BOB(&base_termite);
	Destroy_BOB(&base_scorpion);
	Destroy_BOB(&base_spiderweb);
	Destroy_BOB(&base_honey);
	Destroy_BOB(&base_badger);
	Destroy_BOB(&base_wall);
	Destroy_BOB(&explosion);

	
// shutdonw directdraw
DDraw_Shutdown();

// return success
return(1);
} // end Game_Shutdown

///////////////////////////////////////////////////////////

int Game_Main(void *parms, int num_parms)
{
	int type, collisonType; // used to extract bob type info
	int x,y,x0,y0; // used to compute the upper left and lower right corners of each object
	bool collide = false; //Collsion dectection
	int pixels = 5; //Pixel leway on collisions
	char buffer[80]; //Used to draw text

	BOB_PTR object_ptr; // the object that is going to be rendered

	// check of user is trying to exit
	if (KEY_DOWN(VK_ESCAPE) || KEY_DOWN(VK_SPACE))
		PostMessage(main_window_handle, WM_DESTROY,0,0);

	// start the timing clock
	Start_Clock();

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsback, 100);

	if(KEY_DOWN('1')){ //If the player presses '1'
		if(BUGS[0] == BLANK_SPACE){ //No bug? let the player know
			draw_rect(SCREEN_WIDTH / 2 - 110, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
			Draw_Text_GDI("There's no bug assigned to that key!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,255,0), lpddsprimary);
			Sleep(1500);
		}else{ //Otherwise calculate the position in front of you
			switch(direction){
				case UP:
					x = (player.x - currentMaze.maze[0][0].x) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y - CELL_HEIGHT) / CELL_HEIGHT - 2;
					break;
				case DOWN:
					x = (player.x - currentMaze.maze[0][0].x) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y + CELL_HEIGHT) / CELL_HEIGHT - 2;
					break;
				case LEFT:
					x = (player.x - currentMaze.maze[0][0].x - CELL_WIDTH) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y) / CELL_HEIGHT - 2;
					break;
				case RIGHT:
					x = (player.x - currentMaze.maze[0][0].x + CELL_WIDTH) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y) / CELL_HEIGHT - 2;
					break;
			}

			if(!match(BUGS[0], currentMaze.maze[y][x].type)){ //If bug doesn't match obstacle, let the player know
				draw_rect(SCREEN_WIDTH / 2 - 100, SCREEN_WIDTH / 2 + 120, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You can't use that bug here!", SCREEN_WIDTH / 2 - 75, SCREEN_HEIGHT / 2, RGB(0,255,0), lpddsprimary);
				Sleep(1000);
			}else{ //Otherwise, nuke the obstacle
				currentMaze.maze[y][x].type = BLANK_SPACE;
				boom = true;
				Set_Pos_BOB(&explosion, currentMaze.maze[y][x].x - world_x, currentMaze.maze[y][x].y - world_y);
				Set_Animation_BOB(&explosion, 0);
				Sleep(300);
			}
		}
	}else if(KEY_DOWN('2')){//If the player presses '1'
		if(BUGS[1] == BLANK_SPACE){//No bug? let the player know
			draw_rect(SCREEN_WIDTH / 2 - 110, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
			Draw_Text_GDI("There's no bug assigned to that key!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,255,0), lpddsprimary);
			Sleep(1500);
		}else{//Otherwise calculate the position in front of you
			switch(direction){
				case UP:
					x = (player.x - currentMaze.maze[0][0].x) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y - CELL_HEIGHT) / CELL_HEIGHT - 2;
					break;
				case DOWN:
					x = (player.x - currentMaze.maze[0][0].x) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y + CELL_HEIGHT) / CELL_HEIGHT - 2;
					break;
				case LEFT:
					x = (player.x - currentMaze.maze[0][0].x - CELL_WIDTH) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y) / CELL_HEIGHT - 2;
					break;
				case RIGHT:
					x = (player.x - currentMaze.maze[0][0].x + CELL_WIDTH) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y) / CELL_HEIGHT - 2;
					break;
			}
			if(!match(BUGS[1], currentMaze.maze[y][x].type)){//If bug doesn't match obstacle, let the player know
				draw_rect(SCREEN_WIDTH / 2 - 100, SCREEN_WIDTH / 2 + 120, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You can't use that bug here!", SCREEN_WIDTH / 2 - 75, SCREEN_HEIGHT / 2, RGB(0,255,0), lpddsprimary);
				Sleep(1000);
			}else{//Otherwise, nuke the obstacle
				currentMaze.maze[y][x].type = BLANK_SPACE;
				boom = true;
				Set_Pos_BOB(&explosion, currentMaze.maze[y][x].x - world_x, currentMaze.maze[y][x].y - world_y);
				Set_Animation_BOB(&explosion, 0);
				Sleep(300);
			}
		}
	}else if(KEY_DOWN('3')){//If the player presses '3'
		if(BUGS[2] == BLANK_SPACE){//No bug? let the player know
			draw_rect(SCREEN_WIDTH / 2 - 110, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
			Draw_Text_GDI("There's no bug assigned to that key!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,255,0), lpddsprimary);
			Sleep(1500);
		}else{//Otherwise calculate the position in front of you
			switch(direction){
				case UP:
					x = (player.x - currentMaze.maze[0][0].x) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y - CELL_HEIGHT) / CELL_HEIGHT - 2;
					break;
				case DOWN:
					x = (player.x - currentMaze.maze[0][0].x) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y + CELL_HEIGHT) / CELL_HEIGHT - 2;
					break;
				case LEFT:
					x = (player.x - currentMaze.maze[0][0].x - CELL_WIDTH) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y) / CELL_HEIGHT - 2;
					break;
				case RIGHT:
					x = (player.x - currentMaze.maze[0][0].x + CELL_WIDTH) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y) / CELL_HEIGHT - 2;
					break;
			}
			if(!match(BUGS[2], currentMaze.maze[y][x].type)){//If bug doesn't match obstacle, let the player know
				draw_rect(SCREEN_WIDTH / 2 - 100, SCREEN_WIDTH / 2 + 120, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You can't use that bug here!", SCREEN_WIDTH / 2 - 75, SCREEN_HEIGHT / 2, RGB(0,255,0), lpddsprimary);
				Sleep(1000);
			}else{//Otherwise, nuke the obstacle
				currentMaze.maze[y][x].type = BLANK_SPACE;
				boom = true;
				Set_Pos_BOB(&explosion, currentMaze.maze[y][x].x - world_x, currentMaze.maze[y][x].y - world_y);
				Set_Animation_BOB(&explosion, 0);
				Sleep(300);
			}
		}
	}else if(KEY_DOWN('4')){//If the player presses '4'
		if(BUGS[3] == BLANK_SPACE){//No bug? let the player know
			draw_rect(SCREEN_WIDTH / 2 - 110, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
			Draw_Text_GDI("There's no bug assigned to that key!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,255,0), lpddsprimary);
			Sleep(1500);
		}else{//Otherwise calculate the position in front of you
			switch(direction){
				case UP:
					x = (player.x - currentMaze.maze[0][0].x) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y - CELL_HEIGHT) / CELL_HEIGHT - 2;
					break;
				case DOWN:
					x = (player.x - currentMaze.maze[0][0].x) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y + CELL_HEIGHT) / CELL_HEIGHT - 2;
					break;
				case LEFT:
					x = (player.x - currentMaze.maze[0][0].x - CELL_WIDTH) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y) / CELL_HEIGHT - 2;
					break;
				case RIGHT:
					x = (player.x - currentMaze.maze[0][0].x + CELL_WIDTH) / CELL_WIDTH - 5;
					y = (player.y - currentMaze.maze[0][0].y) / CELL_HEIGHT - 2;
					break;
			}
			if(!match(BUGS[3], currentMaze.maze[y][x].type)){//If bug doesn't match obstacle, let the player know
				draw_rect(SCREEN_WIDTH / 2 - 100, SCREEN_WIDTH / 2 + 120, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You can't use that bug here!", SCREEN_WIDTH / 2 - 75, SCREEN_HEIGHT / 2, RGB(0,255,0), lpddsprimary);
				Sleep(1000);
			}else{//Otherwise, nuke the obstacle
				currentMaze.maze[y][x].type = BLANK_SPACE;
				boom = true;
				Set_Pos_BOB(&explosion, currentMaze.maze[y][x].x - world_x, currentMaze.maze[y][x].y - world_y);
				Set_Animation_BOB(&explosion, 0);
				Sleep(300);
			}
		}
	}

	// check for movement
	if (KEY_DOWN(VK_RIGHT)){

		if(!update || direction != RIGHT){ //Standing checks
			Set_Animation_BOB(&player, 7);
			direction = RIGHT;
		}

		collide = false; //Do collision detection
		for(int a = 0; a < currentMaze.ysize; a++)
			for(int b = 0; b < currentMaze.xsize; b++)
				if(currentMaze.maze[a][b].type != PLAYER_OBJ){
					currentMaze.maze[a][b].x -= MOVE_SPEED_X;
					
					if(((player.y + pixels > currentMaze.maze[a][b].y - world_y && player.y + pixels < currentMaze.maze[a][b].y + CELL_HEIGHT - world_y) || 
						(player.y - pixels + CELL_HEIGHT > currentMaze.maze[a][b].y - world_y && player.y + CELL_HEIGHT - pixels < currentMaze.maze[a][b].y + CELL_HEIGHT - world_y)) &&
						(player.x + CELL_WIDTH > currentMaze.maze[a][b].x - world_x && player.x + CELL_WIDTH < currentMaze.maze[a][b].x - world_x + CELL_WIDTH)){
						if(!collide){
							collisonType = currentMaze.maze[a][b].type;
							if(collisonType != ROCK_OBJ && collisonType != WALL_OBJ && collisonType != HONEY_OBJ)
								currentMaze.maze[a][b].type = BLANK_SPACE;
						}
						collide = true;
	
					}
				}

		if(collide) //If collision
			if(collisonType == ROCK_OBJ || collisonType == WALL_OBJ || collisonType == HONEY_OBJ){
				moveWorldX(MOVE_SPEED_X);
			}else if(collisonType == BADGER_OBJ){ //Badger == death
				draw_rect(SCREEN_WIDTH / 2 - 55, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You got mauled by a badger!", SCREEN_WIDTH / 2 - 50, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				Sleep(1000);
				lose_life();
			}else if(collisonType == SPIDER_WEB_OBJ){ //Spiderweb == death
				draw_rect(SCREEN_WIDTH / 2 - 110, SCREEN_WIDTH / 2 + 130, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You got trapped by the spiderweb!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				Sleep(1000);
				lose_life();
			}else if(collisonType == DOOR_OBJ){ //Door == next level
				if(mazeNumber < 3){
					draw_rect(SCREEN_WIDTH / 2 - 80, SCREEN_WIDTH / 2 + 100, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
					Draw_Text_GDI("On to the next level!", SCREEN_WIDTH / 2 - 60, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				}
				Sleep(1000);
				mazeNumber++;
				LoadMaze();
			}else if(collisonType != BLANK_SPACE){ //Otherwise it's a bug and add it in
				addBug(collisonType);
				printBugInfo(collisonType);
			}
			
		update = true;

	}else if (KEY_DOWN(VK_LEFT)){

		if(!update || direction != LEFT){//Standing checks
			Set_Animation_BOB(&player, 5);
			direction = LEFT;
		}

		collide = false;
		for(int a = 0; a < currentMaze.ysize; a++)//Do collision detection
			for(int b = 0; b < currentMaze.xsize; b++){
				if(currentMaze.maze[a][b].type != PLAYER_OBJ)
					currentMaze.maze[a][b].x += MOVE_SPEED_X;
					
					if(((player.y + pixels > currentMaze.maze[a][b].y - world_y && player.y + pixels < currentMaze.maze[a][b].y + CELL_HEIGHT - world_y) || 
						(player.y - pixels + CELL_HEIGHT > currentMaze.maze[a][b].y - world_y && player.y + CELL_HEIGHT - pixels < currentMaze.maze[a][b].y + CELL_HEIGHT - world_y)) &&
						(player.x > currentMaze.maze[a][b].x - world_x && player.x < currentMaze.maze[a][b].x - world_x + CELL_WIDTH)){
						
						if(!collide){
							collisonType = currentMaze.maze[a][b].type;
							if(collisonType != ROCK_OBJ && collisonType != WALL_OBJ && collisonType != HONEY_OBJ)
								currentMaze.maze[a][b].type = BLANK_SPACE;
							}
						collide = true;
					}


				}

		if(collide)//If collision
			if(collisonType == ROCK_OBJ || collisonType == WALL_OBJ || collisonType == SPIDER_WEB_OBJ || collisonType == BADGER_OBJ|| collisonType == HONEY_OBJ){
				moveWorldX(-MOVE_SPEED_X);
			}else if(collisonType == BADGER_OBJ){//Badger == death
				draw_rect(SCREEN_WIDTH / 2 - 55, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You got mauled by a badger!", SCREEN_WIDTH / 2 - 50, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				Sleep(1000);
				lose_life();
			}else if(collisonType == SPIDER_WEB_OBJ){//Spiderweb == death
				draw_rect(SCREEN_WIDTH / 2 - 110, SCREEN_WIDTH / 2 + 130, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You got trapped by the spiderweb!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				Sleep(1000);
				lose_life();
			}else if(collisonType == DOOR_OBJ){//Door == next level
				if(mazeNumber < 3){
					draw_rect(SCREEN_WIDTH / 2 - 80, SCREEN_WIDTH / 2 + 100, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
					Draw_Text_GDI("On to the next level!", SCREEN_WIDTH / 2 - 60, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				}
				Sleep(1000);
				mazeNumber++;
				LoadMaze();
			}else if(collisonType != BLANK_SPACE){//Otherwise it's a bug and add it in
				addBug(collisonType);
				printBugInfo(collisonType);
			}

		update = true;

	}else if (KEY_DOWN(VK_UP)){

		if(!update || direction != UP){//Standing checks
			Set_Animation_BOB(&player, 6);
			direction = UP;
		}

		collide = false;
		for(int a = 0; a < currentMaze.ysize; a++)//Do collision detection
			for(int b = 0; b < currentMaze.xsize; b++){
				if(currentMaze.maze[a][b].type == BLANK_SPACE)
					continue;

				currentMaze.maze[a][b].y += MOVE_SPEED_Y;

				if(((player.x + pixels > currentMaze.maze[a][b].x - world_x && player.x + pixels < currentMaze.maze[a][b].x + CELL_WIDTH - world_x) || 
						(player.x - pixels + CELL_WIDTH > currentMaze.maze[a][b].x - world_x && player.x + CELL_WIDTH - pixels < currentMaze.maze[a][b].x + CELL_WIDTH - world_x)) &&
						(player.y < currentMaze.maze[a][b].y - world_y + CELL_HEIGHT && player.y > currentMaze.maze[a][b].y - world_y)){
						if(!collide){
							collisonType = currentMaze.maze[a][b].type;
							if(collisonType != ROCK_OBJ && collisonType != WALL_OBJ && collisonType != HONEY_OBJ)
								currentMaze.maze[a][b].type = BLANK_SPACE;
							}
						collide = true;
					}
			}

		if(collide)//If collision
			if(collisonType == ROCK_OBJ || collisonType == WALL_OBJ || collisonType == HONEY_OBJ){
				moveWorldY(-MOVE_SPEED_Y);
			}else if(collisonType == BADGER_OBJ){//Badger == death
				draw_rect(SCREEN_WIDTH / 2 - 55, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You got mauled by a badger!", SCREEN_WIDTH / 2 - 50, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				Sleep(1000);
				lose_life();
			}else if(collisonType == SPIDER_WEB_OBJ){//Spiderweb == death
				draw_rect(SCREEN_WIDTH / 2 - 110, SCREEN_WIDTH / 2 + 130, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You got trapped by the spiderweb!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				Sleep(1000);
				lose_life();
			}else if(collisonType == DOOR_OBJ){//Door == next level
				if(mazeNumber < 3){
					draw_rect(SCREEN_WIDTH / 2 - 80, SCREEN_WIDTH / 2 + 100, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("On to the next level!", SCREEN_WIDTH / 2 - 60, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				}
				Sleep(1000);
				mazeNumber++;
				LoadMaze();
			}else if(collisonType != BLANK_SPACE){//Otherwise it's a bug and add it in
				addBug(collisonType);
				printBugInfo(collisonType);
			}

		update = true;

	}else if (KEY_DOWN(VK_DOWN)){

		if(!update || direction != DOWN){//Standing checks
			Set_Animation_BOB(&player, 4);
			direction = DOWN;
		}

		collide = false;
		for(int a = 0; a < currentMaze.ysize; a++)//Do collision detection
			for(int b = 0; b < currentMaze.xsize; b++){
				if(currentMaze.maze[a][b].type != PLAYER_OBJ)
					currentMaze.maze[a][b].y -= MOVE_SPEED_Y;

				if(((player.x + pixels > currentMaze.maze[a][b].x - world_x && player.x + pixels < currentMaze.maze[a][b].x + CELL_WIDTH - world_x) || 
						(player.x - pixels + CELL_WIDTH > currentMaze.maze[a][b].x - world_x && player.x + CELL_WIDTH - pixels < currentMaze.maze[a][b].x + CELL_WIDTH - world_x)) &&
						(player.y + CELL_HEIGHT < currentMaze.maze[a][b].y - world_y + CELL_HEIGHT && player.y + CELL_HEIGHT > currentMaze.maze[a][b].y - world_y)){
						if(!collide){
							collisonType = currentMaze.maze[a][b].type;
							if(collisonType != ROCK_OBJ && collisonType != WALL_OBJ && collisonType != HONEY_OBJ)
								currentMaze.maze[a][b].type = BLANK_SPACE;
							}
						collide = true;
					}
			}

		if(collide)//If collision
			if(collisonType == ROCK_OBJ || collisonType == WALL_OBJ || collisonType == HONEY_OBJ){
				moveWorldY(MOVE_SPEED_Y);
			}else if(collisonType == BADGER_OBJ){//Badger == death
				draw_rect(SCREEN_WIDTH / 2 - 55, SCREEN_WIDTH / 2 + 140, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You got mauled by a badger!", SCREEN_WIDTH / 2 - 50, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				Sleep(1000);
				lose_life();
			}else if(collisonType == SPIDER_WEB_OBJ){//Spiderweb == death
				draw_rect(SCREEN_WIDTH / 2 - 110, SCREEN_WIDTH / 2 + 130, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
				Draw_Text_GDI("You got trapped by the spiderweb!", SCREEN_WIDTH / 2 - 100, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				Sleep(1000);
				lose_life();
			}else if(collisonType == DOOR_OBJ){//Door == next level
				if(mazeNumber < 3){
					draw_rect(SCREEN_WIDTH / 2 - 80, SCREEN_WIDTH / 2 + 100, SCREEN_HEIGHT / 2, SCREEN_HEIGHT / 2 + 20, 255);
					Draw_Text_GDI("On to the next level!", SCREEN_WIDTH / 2 - 60, SCREEN_HEIGHT / 2, RGB(0,0,255), lpddsprimary);
				}
				Sleep(1000);
				mazeNumber++;
				LoadMaze();
			}else if(collisonType != BLANK_SPACE){//Otherwise it's a bug and add it in
				addBug(collisonType);
				printBugInfo(collisonType);
			}

		update = true;

	}else{ //This is the switch for a standing character
		switch(direction){
			case DOWN:
				Set_Animation_BOB(&player, 0);
				Animate_BOB(&player);
				break;
			case LEFT:
				Set_Animation_BOB(&player, 1);
				Animate_BOB(&player);
				break;
			case UP:
				Set_Animation_BOB(&player, 2);
				Animate_BOB(&player);
				break;
			case RIGHT:
				Set_Animation_BOB(&player, 3);
				Animate_BOB(&player);
				break;
			default:
				break;
		}

		update = false;
	}
		
	for(int a = 0; a < currentMaze.ysize; a++)
		for(int b = 0; b < currentMaze.xsize; b++){
			// get the object position
			x0 = currentMaze.maze[a][b].x;
			y0 = currentMaze.maze[a][b].y;

			// now translate to screen coords, could have done above, but clearer
			x0 -= world_x;
			y0 -= world_y;

			if(x0 > SCREEN_WIDTH || y0 > SCREEN_HEIGHT || x0 + CELL_WIDTH < 0 || y0 + CELL_HEIGHT < 0)
				continue;

			// get type, so we can compute size
			type = currentMaze.maze[a][b].type;
		    
			//put in the proper pointer
			switch(type){
				case PLAYER_OBJ:
					continue;
				case ROCK_OBJ:
				{
					object_ptr = &base_rock;
				}break;
				case FIREFLY_OBJ:
				{
					Set_Pos_BOB(&base_firefly,currentMaze.maze[a][b].x, currentMaze.maze[a][b].y);
					object_ptr = &base_firefly;
				}break;
				case BEE_OBJ:
				{
					Set_Pos_BOB(&base_bee,currentMaze.maze[a][b].x, currentMaze.maze[a][b].y);
					object_ptr = &base_bee;
				}break;
				case SCORPION_OBJ:
				{
					Set_Pos_BOB(&base_scorpion,currentMaze.maze[a][b].x, currentMaze.maze[a][b].y);
					object_ptr = &base_scorpion;
				}break;
				case TERMITE_OBJ:
				{
					Set_Pos_BOB(&base_termite,currentMaze.maze[a][b].x, currentMaze.maze[a][b].y);
					object_ptr = &base_termite;
				}break;
				case DOOR_OBJ:
				{
					Set_Pos_BOB(&base_door,currentMaze.maze[a][b].x, currentMaze.maze[a][b].y);
					object_ptr = &base_door;
				}break;
				case SPIDER_WEB_OBJ:
				{
					Set_Pos_BOB(&base_spiderweb,currentMaze.maze[a][b].x, currentMaze.maze[a][b].y);
					object_ptr = &base_spiderweb;
				}break;
				case BADGER_OBJ:
				{
					Set_Pos_BOB(&base_badger,currentMaze.maze[a][b].x, currentMaze.maze[a][b].y);
					object_ptr = &base_badger;
				}break;
				case HONEY_OBJ:
				{
					Set_Pos_BOB(&base_honey,currentMaze.maze[a][b].x, currentMaze.maze[a][b].y);
					object_ptr = &base_honey;
				}break;
				case WALL_OBJ:
				{
					Set_Pos_BOB(&base_wall,currentMaze.maze[a][b].x, currentMaze.maze[a][b].y);
					object_ptr = &base_wall;
				}break;

				default:continue;

			} // end switch
			
			//Update the BOB position
			object_ptr->x = x0;
			object_ptr->y = y0;
					
			// render the object
			Draw_Scaled_BOB(object_ptr, CELL_WIDTH, CELL_HEIGHT, lpddsback);

		} //End for if

	if(update) //if the player is walking, update the animation
		Animate_BOB(&player);

	if(boom){ //if there's an explosion, draw it
		Draw_Scaled_BOB(&explosion, CELL_WIDTH, CELL_HEIGHT, lpddsback);
		Animate_BOB(&explosion);
		
		if(explosion.anim_index == 2)
			boom = false;
	}
				
	//draw the player
	Draw_Scaled_BOB(&player, CELL_WIDTH, CELL_HEIGHT, lpddsback);

	// flip the surfaces
	DDraw_Flip();

	//Draw the bug hotbar and the hotbar text
	draw_rect(0, SCREEN_WIDTH, SCREEN_HEIGHT - CELL_HEIGHT - 25, SCREEN_HEIGHT, 200);
	sprintf(buffer,"BUG HOTBAR:                                 Lives: %d   Level: %d", lives, mazeNumber);
	Draw_Text_GDI(buffer, 5, SCREEN_HEIGHT - CELL_WIDTH - 45, RGB(0,255,0), lpddsprimary);

	//Draw the bugs into the bug hotbar
	for(int a = 0; a < 8; a++){
		if(BUGS[a] == FIREFLY_OBJ){
			Set_Pos_BOB(&base_firefly, a * CELL_WIDTH,SCREEN_HEIGHT - CELL_HEIGHT);
			Draw_Scaled_BOB(&base_firefly, CELL_WIDTH, CELL_HEIGHT, lpddsprimary);
		}else if(BUGS[a] == BEE_OBJ){
			Set_Pos_BOB(&base_bee, a * CELL_WIDTH,SCREEN_HEIGHT - CELL_HEIGHT);
			Draw_Scaled_BOB(&base_bee, CELL_WIDTH, CELL_HEIGHT, lpddsprimary);
		}else if(BUGS[a] == SCORPION_OBJ){
			Set_Pos_BOB(&base_scorpion, a * CELL_WIDTH,SCREEN_HEIGHT - CELL_HEIGHT);
			Draw_Scaled_BOB(&base_scorpion, CELL_WIDTH, CELL_HEIGHT, lpddsprimary);
		}else if(BUGS[a] == TERMITE_OBJ){
			Set_Pos_BOB(&base_termite, a * CELL_WIDTH,SCREEN_HEIGHT - CELL_HEIGHT);
			Draw_Scaled_BOB(&base_termite, CELL_WIDTH, CELL_HEIGHT, lpddsprimary);
		}else if(BUGS[a] == BLANK_SPACE)
			break;

		sprintf(buffer, "%d", a + 1);
		Draw_Text_GDI(buffer, a * CELL_WIDTH + CELL_WIDTH / 2 - 5, SCREEN_HEIGHT - CELL_WIDTH + 10, RGB(0,255,0), lpddsprimary);

	}

	// sync to 30 fps
	Wait_Clock(30);

	// return success
	return(1);

} // end Game_Main

//////////////////////////////////////////////////////////</div>